﻿local _, L = ...
local mediaFolder = "Interface\\AddOns\\".._.."\\media\\"
local LMB, RMB = CreateAtlasMarkup("NPE_LeftClick", 16, 16), CreateAtlasMarkup("NPE_RightClick", 16,16)

local LSM = LibStub('LibSharedMedia-3.0')

local max = max
local string_match = string.match
local math_floor = math.floor
local string_len = string.len
local string_gsub = string.gsub
local table_insert = table.insert
local table_sort = table.sort

local CloseLoot = CloseLoot
local CreateFrame = CreateFrame
local CursorOnUpdate = CursorOnUpdate
local CursorUpdate = CursorUpdate
local GetCursorPosition = GetCursorPosition
local GetCVarBool = GetCVarBool
local GetLootSlotInfo = GetLootSlotInfo
local GetLootSlotLink = GetLootSlotLink
local GetLootSlotType = GetLootSlotType
local GetItemCount = GetItemCount
local GetItemInfo = GetItemInfo
local GetNumLootItems = GetNumLootItems
local HandleModifiedItemClick = HandleModifiedItemClick
local IsModifiedClick = IsModifiedClick
local LootSlot = LootSlot
local LootSlotHasItem = LootSlotHasItem
local ResetCursor = ResetCursor
local StaticPopup_Hide = StaticPopup_Hide
local MasterLooterFrame_Show = MasterLooterFrame_Show
local MasterLooterFrame_UpdatePlayers = MasterLooterFrame_UpdatePlayers
local GetContainerNumFreeSlots = GetContainerNumFreeSlots
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local C_Timer_After = C_Timer.After
local ITEM_QUALITY_COLORS = ITEM_QUALITY_COLORS

local info_frame_scale	= 2
local info_frame_space	= 5
local info_frame_width	= 120

alootFrameMixin = {}
aLootFrameSlotMixin = {}

local function MultiCheck(check, ...)
	for i = 1, select("#", ...) do
		if check == select(i, ...) then return true end  
	end  
	return false  
end
							
local function CreateMyText(self, font, fontsize, outline, layer, align, justifyH, shadow)
	local text = self:CreateFontString(nil, layer or "OVERLAY", nil, 7)
		text:SetFont(LSM:Fetch(LSM.MediaType.FONT, font) or STANDARD_TEXT_FONT, fontsize or 14, outline or "OUTLINE")
		text:SetPoint(align or "CENTER")
		text:SetJustifyH(justifyH or "CENTER")
		text:SetText(nil)
		text:SetHeight(text:GetStringHeight())
		if shadow then
			text:SetShadowColor(0, 0, 0, .6)
			text:SetShadowOffset(1, -1)
		end	
	return text
end	

-- Invert gold patterns
local function invert(pstr)
	return pstr:gsub("%%d", "(%1+)")
end

-- Parse coin strings
local function ParseCoinString(tstr)
	local g = tstr:match(invert(GOLD_AMOUNT)) or 0
	local s = tstr:match(invert(SILVER_AMOUNT)) or 0
	local c = tstr:match(invert(COPPER_AMOUNT)) or 0
	return g*10000+s*100+c
end

function GetMoneyShortString(money)
	local gold = floor(money / (COPPER_PER_SILVER * SILVER_PER_GOLD));
	local silver = floor((money - (gold * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER);
	local copper = mod(money, COPPER_PER_SILVER);
	
	if gold > 0 then
		return "|cFFffff00"..gold..GOLD_AMOUNT_SYMBOL.."|r";
	elseif silver > 0 then
		return "|cFFffffff"..silver..SILVER_AMOUNT_SYMBOL.."|r";
	elseif copper > 0 then
		return "|cFF808080"..copper..COPPER_AMOUNT_SYMBOL.."|r";
	else
		return nil
	end
end

local function SetFrameScale(frame, width, height, from, to)
	if (frame.currentScale ~= to) then
		frame:SetSize(width * from, height * from)
		if (frame.scale:IsPlaying()) then
			frame.scale:Stop()
		end
		frame.scale.width:SetChange(width * to)
		frame.scale.height:SetChange(height * to)
		frame.scale:Play()
		frame.currentScale = to
	end
end

local function ResetInfoFrameFunction(framePool, frame)
	if frame.AnimOut:IsPlaying() then
		frame.AnimOut:Stop()
	end
	if frame.AnimIn:IsPlaying() then
		frame.AnimIn:Stop()
	end
	if frame.AnimDown:IsPlaying() then
		frame.AnimDown:Stop()
	end
	frame:SetAlpha(0)
	frame:Hide()
	frame:SetParent(aLootInfoFrame)
	frame:ClearAllPoints()
	frame:SetPoint("TOP", aLootInfoFrame, "TOP", 0, 0)
	--frame.Text:ClearAllPoints()
	--frame.Text:SetPoint("RIGHT", frame, "RIGHT", -13, 0)
	frame.AnimIn.Alpha:SetDuration(L.config.info_frame_speed)
	frame.AnimDown.Translation:SetDuration(L.config.info_frame_speed)
	frame.createdAt = nil
	frame.expired = false
end

function alootFrameMixin:AddFrameToPool(icon, text, fast)
	local lastFrame = aLootInfoFrame.infoFrameActiveFrames.last
	
	if lastFrame ~= nil and lastFrame.AnimIn:IsPlaying() then
		--print("Neskor ".. text)
		C_Timer_After(lastFrame.AnimIn:GetDuration() + ((L.config.info_frame_speed/2 + .1) * aLootInfoFrame.inFront), function()
			if lastFrame ~= nil then
				alootFrameMixin:AddFrameToPool(icon, text, true)
			end
		end)
		aLootInfoFrame.inFront = aLootInfoFrame.inFront + 1
		return
	end
	if aLootInfoFrame.inFront > 0 then
		aLootInfoFrame.inFront = aLootInfoFrame.inFront - 1
	end
	--print(aLootInfoFrame.inFront, text)
	aLootInfoFrame:Show()
	aLootInfoFrame:EnableMouse(false)
	
	-- hide moveable frame ak je otvoreny
	aLootInfoFrame.DragBar:EnableMouse(false)
	if L.config.unlock_infoFrame then
		aLootInfoFrame.Text:SetText(nil)
		aLootInfoFrame:SetBackdropColor(0, 0, 0, 0)
		aLootInfoFrame:SetBackdropBorderColor(0, 0, 0, 0)
		L.config.unlock_infoFrame = false
		alootFrameMixin:UpdateNotifyChange("aLootInfoFrame", false)
	end
	
	aLootInfoFrame:SetHeight(L.config.info_frame_height + 10)
	
	if aLootInfoFrame:GetScript("OnUpdate") == nil then
		aLootInfoFrame:SetScript("OnUpdate", function(self, elapsed)
			if self.infoFrameActiveFrames:GetNumActive() > 0 then
				self:Show()
				
				local sortedFrames = {}
				for frame in self.infoFrameActiveFrames:EnumerateActive() do
					table_insert(sortedFrames, frame)
				end
				table_sort(sortedFrames, function(a, b) return a.createdAt < b.createdAt end)
				
				for _, frame in ipairs(sortedFrames) do
					if GetTime() >= (frame.createdAt + L.config.info_frame_duration) and not (frame.AnimOut:IsPlaying() or frame.expired) then
						frame.AnimOut:Play()
						frame.AnimOut:SetScript("OnFinished", function(self)
							self:GetParent().expired = true
						end)
					end
					
					if self.infoFrameActiveFrames:GetNumActive() > L.config.info_frame_maxcount or frame.expired then
						frame.Text:SetText(nil)
						frame.Text:Hide()
						frame:Hide()
						self.infoFrameActiveFrames:Release(frame)
					end
				end
			else
				aLootInfoFrame.infoFrameActiveFrames:ReleaseAll()
				aLootInfoFrame.infoFrameActiveFrames.last = nil
				aLootInfoFrame.inFront = 0
				if not L.config.unlock_infoFrame then
					aLootInfoFrame:Hide()
				end
				aLootInfoFrame:SetScript("OnUpdate", nil)
			end
		end)
	end
	
	local newFrame = aLootInfoFrame.infoFrameActiveFrames:Acquire()
	newFrame:SetPoint("BOTTOMLEFT", aLootInfoFrame, "BOTTOMLEFT", 0, 0)
	newFrame:EnableMouse(false)
	newFrame.createdAt = GetTime()
	
	newFrame.IconFrame:SetSize(L.config.info_frame_height + 20, L.config.info_frame_height + 20)	
	newFrame.IconFrame.Icon:SetSize(L.config.info_frame_height, L.config.info_frame_height)
	newFrame.IconFrame.Icon:SetTexture(icon)
		
	--newFrame.Text:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.info_frame_font) or STANDARD_TEXT_FONT, L.config.info_frame_fontsize or 14, L.config.info_frame_font_flag or "OUTLINE")
	--newFrame.Text:SetPoint("RIGHT", newFrame, "RIGHT", -13, 0)
	--newFrame.Text:SetText(text)
	--newFrame.Text:Show()
		
	newFrame.Text = CreateMyText(newFrame, L.config.info_frame_font, L.config.info_frame_fontsize, L.config.info_frame_font_flag, "OVERLAY", "RIGHT", "CENTER", true)
	newFrame.Text:SetPoint("RIGHT", newFrame, "RIGHT", -13, 0)
	newFrame.Text:SetText(text)
	
	newFrame.currentScale = info_frame_scale
	newFrame.scale = CreateAnimationGroup(newFrame)
	newFrame.scale.width = newFrame.scale:CreateAnimation("Width")
	newFrame.scale.width:SetDuration(L.config.info_frame_speed)
	newFrame.scale.height = newFrame.scale:CreateAnimation("Height")
	newFrame.scale.height:SetDuration(L.config.info_frame_speed)
			
	-- AnimeIn
	if fast then
		newFrame.AnimIn.Alpha:SetDuration(L.config.info_frame_speed / 2)
		newFrame.scale.width:SetDuration(L.config.info_frame_speed/2)
		newFrame.scale.height:SetDuration(L.config.info_frame_speed/2)
	end
		
	SetFrameScale(newFrame, newFrame.Text:GetStringWidth() + L.config.info_frame_height + 40, L.config.info_frame_height + 10, info_frame_scale, 1)
	
	--newFrame.AnimIn.Alpha:SetFromAlpha(0)
	newFrame.AnimIn.Alpha:SetToAlpha(L.config.info_frame_alpha)
	newFrame.AnimIn:Play()
	
	newFrame.AnimIn:SetScript("OnFinished", function(self)
		self:GetParent():SetAlpha(L.config.info_frame_alpha)
	end)
	
	if newFrame.AnimIn:IsPlaying() then
		newFrame.AnimIn:GetParent():Show()
	end

	if L.config.info_frame_flash then
		--newFrame.IconFrame.glow:SetVertexColor(1, 1, 1, 1)
		--newFrame.IconFrame.glow:Show()
		newFrame.IconFrame.glow.animIn:Play()
	end
	--newFrame.glow:Show()
	--newFrame.glow.animIn:Play()
	
	newFrame.AnimOut.Alpha:SetFromAlpha(L.config.info_frame_alpha)
	--newFrame.AnimOut.Alpha:SetToAlpha(0)	
	
	-- Anime move
	if lastFrame ~= nil and lastFrame ~= newFrame and not lastFrame.AnimOut:IsPlaying() then
		if fast then
			lastFrame.AnimDown.Translation:SetDuration(L.config.info_frame_speed / 2)
		end
		lastFrame.AnimDown.Translation:SetOffset(0, - L.config.info_frame_height - (info_frame_space + 2))
		lastFrame.AnimDown:Play()
		lastFrame.AnimDown:SetScript("OnFinished", function(self)
			self:GetParent():ClearAllPoints()
			self:GetParent():SetParent(newFrame)
			self:GetParent():SetPoint("BOTTOMLEFT", newFrame, "BOTTOMLEFT", 0, - L.config.info_frame_height - (info_frame_space + 2))
			self:GetParent():SetAlpha(1)
		end)
	end

	aLootInfoFrame.infoFrameActiveFrames.last = newFrame
end

function alootFrameMixin:ShowFlashFrame(icon, lootName)
	if aLootFlashFrame.Anime:IsPlaying() then
		aLootFlashFrame.Anime:Stop()
	end
		
	aLootFlashFrame.DragBar:EnableMouse(false)
	aLootFlashFrame.Text:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.flash_frame_font) or STANDARD_TEXT_FONT, L.config.flash_frame_fontsize or 14, L.config.flash_frame_font_flag or "OUTLINE")
	aLootFlashFrame.Text:SetText(lootName)
	aLootFlashFrame:SetSize(aLootFlashFrame.Text:GetStringWidth() + 90, L.config.flash_frame_height)
	aLootFlashFrame.IconFrame.Icon:SetTexture(icon or 939375)
	aLootFlashFrame:Show()
	aLootFlashFrame.Anime.AlphaIn:SetToAlpha(L.config.flash_frame_alpha)
	aLootFlashFrame.Anime:Play()
	
	if L.config.flash_frame_flash then
		aLootFlashFrame.shine.animIn:Play()
		aLootFlashFrame.glow:SetVertexColor(.5, .5, .5, 1)
		aLootFlashFrame.glow.animIn:Play()
	end
end


local function OnLootClosed()
	if aLootFrame.lootReadyIsOpened then
		aLootFrame.lootReadyIsOpened = false
		
		if aLootFrame:IsShown() then
			StaticPopup_Hide('LOOT_BIND')
			aLootFrame:Hide()
		
			-- pre istotu Hide na vsetky sloty
			for _, frame in pairs(aLootFrame.slots) do
				frame:Hide()
			end
		end		
	
		if L.config.infoFrame and L.config.info_frame_showBagSpace then
			local totalFree = CalculateTotalNumberOfFreeBagSlots()
			if totalFree == 0 then
				alootFrameMixin:AddFrameToPool(133670, "|cffff0000"..ERR_INV_FULL.."|r")
			elseif totalFree == 1 then
				alootFrameMixin:AddFrameToPool(133670, "|cffff0000Last free slot|r")
			elseif totalFree < L.config.info_frame_numBagSpace then
				alootFrameMixin:AddFrameToPool(133670, "|cffff0000"..totalFree.." free slots in bag|r")
			end
		end
		
		if not aLootFrame.isFromItem and not aLootFrame.autoLoot then
			CloseAllBags()
		end
		
		--alootFrameMixin:ShowFlashFrame(nil, aLootFrame.autoLoot and "autoLoot |cFF00FF00TRUE" or "autoLoot |cFFFF0000FALSE")
	end
end

local function OnLootReady(autoloot)
	if not aLootFrame.lootReadyIsOpened then
		aLootFrame.numLootItems = GetNumLootItems()
		aLootFrame.lootReadyIsOpened = true
				
		if aLootFrame.numLootItems > 0 then
			for slotIndex = aLootFrame.numLootItems, 1, -1 do
				LootSlot(slotIndex)
			end

			if L.config.infoFrame and  L.config.info_frame_speedLoot_text then
				alootFrameMixin:AddFrameToPool(133784, "|cff00bbffLooted|r "..aLootFrame.numLootItems.." |cff00bbffitems|r")
			end
		end
	end
end

-- uprav zarovnanie pri zmene slot itemov
local function anchorSlots()
	local shownSlots, firstSlot, lastSlot = 0
	
	for i = 1, #aLootFrame.slots do
		aLootFrame.slots[i]:ClearAllPoints()
		if aLootFrame.slots[i]:IsShown() then
			shownSlots = shownSlots + 1
			if not firstSlot then
				firstSlot = i
			end
			if shownSlots > 1 then
				if L.config.lootFrame_align == "LEFT" then
					aLootFrame.slots[i]:SetPoint("LEFT", lastSlot, "RIGHT", L.config.lootFrame_iconSpace - 4, 0)
				elseif L.config.lootFrame_align == "RIGHT" then
					aLootFrame.slots[i]:SetPoint("RIGHT", lastSlot, "LEFT", - L.config.lootFrame_iconSpace + 4, 0)
				elseif L.config.lootFrame_align == "TOP" then
					aLootFrame.slots[i]:SetPoint("TOP", lastSlot, "BOTTOM", 0, - L.config.lootFrame_iconSpace + 4)
				elseif L.config.lootFrame_align == "BOTTOM" then
					aLootFrame.slots[i]:SetPoint("BOTTOM", lastSlot, "TOP", 0, L.config.lootFrame_iconSpace - 4)
				else
					aLootFrame.slots[i]:SetPoint("LEFT", lastSlot, "RIGHT", L.config.lootFrame_iconSpace - 4, 0)
				end
			end
			lastSlot = aLootFrame.slots[i]
		end
	end
	
	if shownSlots > 0 then
		if C_CVar.GetCVarBool("lootUnderMouse") or L.config.lootFrame_align ~= "CENTER" then
			aLootFrame.slots[firstSlot]:SetPoint("CENTER", aLootFrame, "CENTER", 0, 0) --L.config.lootFrame_align, aLootFrame, L.config.lootFrame_align, 0, 0)
		else
			aLootFrame.slots[firstSlot]:SetPoint("CENTER", aLootFrame, "CENTER", -(((L.config.lootFrame_iconSize + L.config.lootFrame_iconSpace) * (shownSlots - 1)) / 2), 0)
		end
	end
end

local function createSlot(id)
	local frame = CreateFrame("Button", 'aLootFrameSlot'..id, aLootFrame, "aLootFrameSlot")
		frame:SetFrameLevel(aLootFrame:GetFrameLevel() + 1)
		frame:SetSize(L.config.lootFrame_iconSize, L.config.lootFrame_iconSize)
		frame:SetID(id)
						
		frame.scale = CreateAnimationGroup(frame)
		frame.scale.width = frame.scale:CreateAnimation("Width")
		frame.scale.width:SetDuration(.3)
		frame.scale.height = frame.scale:CreateAnimation("Height")
		frame.scale.height:SetDuration(.3)
	
		frame.nameTexture:SetTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, L.config.name_texture))
		frame.name:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.name_font), L.config.name_fontsize,L.config.name_font_flag)
		frame.count:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.name_font), L.config.name_fontsize,L.config.name_font_flag)
		--frame.sellPrice:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.name_font), L.config.name_fontsize,L.config.name_font_flag)
		
	aLootFrame.slots[id] = frame
	return frame
end

local function CheckFreeSpace(itemLink, lootQuantity)
	local itemFamily = GetItemFamily(itemLink)
	local itemCount = GetItemCount(itemLink)
	
	--[[for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(bagID)
		for slot = 1, numSlots do
			local itemLoc = ItemLocation:CreateFromBagAndSlot(bagID, slot)
			if itemLoc:IsValid() then
				local id = C_Item.GetItemID(itemLoc)
				local isUnique, limitCategoryName, limitCategoryCount, limitCategoryID = C_Item.GetItemUniquenessByID(id)
				if isUnique then
					print(limitCategoryName, limitCategoryCount, limitCategoryID)
				end
			end
		end
	end]]--
	
	
	for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local numberOfFreeSlots, bagType = GetContainerNumFreeSlots(bagID)
		if (not bagType or bagType == 0) or (itemFamily and bit.band(itemFamily, bagType) > 0) then
			if numberOfFreeSlots > 0 then
				return true, numberOfFreeSlots, itemCount
			end
		end
	end
	
	if itemCount > 0 then
		local itemStackCount = select(8, GetItemInfo(itemLink))
		if itemStackCount > 1 then
			local remainingSpace = (itemStackCount - itemCount) % itemStackCount
			if remainingSpace >= lootQuantity then
				return true, numberOfFreeSlots, itemCount
			end
		end
	end

	return false, 0, 0
end

local function OpenSlot(slotIndex, openTestSlot, showSlot)
	if LootSlotHasItem(slotIndex) or openTestSlot then
		--[[
		itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID or "itemString" or "itemName" or "slot.itemLink")
		slot.lootName, slot.lootIcon, slot.lootQuantity, slot.lootQuality = CurrencyContainerUtil.GetCurrencyContainerInfo(slot.currencyID, slot.lootQuantity, slot.lootName, slot.lootIcon, slot.lootQuality)
		slot.lootQuality = 0 Poor, 1 Common, 2 Uncommon, 3 Rare, 4 Epic, 5 Legendary, 6 Artifact
		slotType 0: LOOT_SLOT_NONE, 1: LOOT_SLOT_ITEM, 2: LOOT_SLOT_MONEY, 3: LOOT_SLOT_CURRENCY
		]]--
		local isFreeSpaceInBags, numberOfFreeSlots
		local slot = aLootFrame.slots[slotIndex] or createSlot(slotIndex)
		
		slot.currentScale = 1
		slot.questTexture:Hide()
		slot.opened = true
		slot.itemSellPrice = nil
		
		if openTestSlot then
			slot.lootIcon, slot.lootName, slot.lootQuantity, slot.currencyID, slot.lootQuality, slot.locked, slot.isQuestItem, slot.questID, slot.isActive = 939375, "Test", 3, nil, 1, false, false, nil, nil
			slot.slotType = 1
			slot.itemLink = select(2, GetItemInfo(slot.lootName))
			slot.itemSellPrice = 1212122
			aLootFrame.numLootItems = 1
		else
			slot.lootIcon, slot.lootName, slot.lootQuantity, slot.currencyID, slot.lootQuality, slot.locked, slot.isQuestItem, slot.questID, slot.isActive = GetLootSlotInfo(slotIndex)
			slot.slotType = GetLootSlotType(slotIndex)	
			slot.itemLink = GetLootSlotLink(slotIndex)		
		end
		
		if slot.locked then
			slot.color = { r = 1, g = 0, b = 0, hex = "|cFFFF0000" }
		elseif (slot.questID and not slot.isActive) or (slot.questID or slot.isQuestItem) then
			slot.color = { r = 1, g = 1, b = 0, hex = "|cFFFFFF00" }
			slot.questTexture:Show()
		elseif slot.lootQuality > 1 then
			slot.color = ITEM_QUALITY_COLORS[slot.lootQuality]
		else
			slot.color = { r = 0, g = 0, b = 0, hex = ITEM_QUALITY_COLORS[slot.lootQuality].hex }
		end
		
		-- openTestSlot loot, goldy, questItems ked to nie je priamo item, a currency sa nelootuje do baglu preto automaticky je miesto
		if openTestSlot or (slot.isQuestItem and not slot.isActive) then
			isFreeSpaceInBags = true
		elseif slot.slotType == 2 then
			isFreeSpaceInBags = true
			slot.lootName = string_gsub(slot.lootName, "\n", " ")
			slot.lootQuantity = ParseCoinString(slot.lootName)
		elseif slot.slotType == 3 then
			local info = C_CurrencyInfo.GetCurrencyInfo(slot.currencyID)
			slot.itemCount = info.quantity
			isFreeSpaceInBags = true
		else
			isFreeSpaceInBags, numberOfFreeSlots, slot.itemCount = CheckFreeSpace(slot.itemLink, slot.lootQuantity)
			slot.itemSellPrice = select(11, GetItemInfo(slot.itemLink))
		end
		
		if isFreeSpaceInBags and not openTestSlot and not showSlot and
			((aLootFrame.autoLoot and not IsModifiedClick("AUTOLOOTTOGGLE")) 
			or (L.config.autoLoot["gold"] and slot.slotType == 2) 
			or (L.config.autoLoot["currency"] and slot.slotType == 3)
			or (L.config.autoLoot["quest"] and (slot.isQuestItem or slot.questID))
			or (L.config.autoLoot["poor"] and slot.lootQuality == 0)
			or (L.config.autoLoot["common"] and slot.lootQuality == 1)
			or (L.config.autoLoot["uncommon"] and slot.lootQuality == 2)
			or (L.config.autoLoot["rare"] and slot.lootQuality == 3)
			or (L.config.autoLoot["epic"] and slot.lootQuality == 4)
			or (L.config.autoLoot["legendary"] and slot.lootQuality == 5)) then
			
			LootSlot(slotIndex)
							
			if L.config.flashFrame 
				and ((L.config.flash["user"] and L.ItemsTable[slot.lootName] and L.ItemsTable[slot.lootName].enable)
				or (L.config.flash["quest"] and (slot.isQuestItem or slot.questID))
				or (L.config.flash["legendary"] and slot.lootQuality == 5)) then
				
				alootFrameMixin:ShowFlashFrame(slot.lootIcon, slot.color.hex .. slot.lootName.."|r")
				PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_sound), "Master")
			end
				
			if L.config.infoFrameBlizzard then
				if slot.slotType == 2 then
					MoneyWonAlertSystem:AddAlert(slot.lootQuantity)
				elseif slot.slotType == 3 then
					LootAlertSystem:AddAlert(slot.itemLink, slot.lootQuantity, nil, nil, nil, true);
				else
					LootAlertSystem:AddAlert(slot.itemLink, slot.lootQuantity, nil, nil, nil, false);
				end
			end
			
			if L.config.infoFrame then
				local text = ''
				if L.config.info_frame_showLootQuantity and slot.lootQuantity and slot.lootQuantity > 1 then
					text = text .. "|cffFFFFFF x"..slot.lootQuantity.."|r"
				end
				if  L.config.info_frame_showItemCount and slot.itemCount and slot.itemCount > 1 and slot.lootQuantity then
					local tmp = slot.itemCount + slot.lootQuantity
					text = text .. " |cff808080("..tmp..")|r"
				end
				alootFrameMixin:AddFrameToPool(slot.lootIcon, slot.color.hex .. slot.lootName  .."|r"..text)
			end
		
			return
		
		elseif (L.config.flashFrame and not isFreeSpaceInBags) then
			if not aLootFlashFrame.Anime:IsPlaying() then
				alootFrameMixin:ShowFlashFrame(nil, "|cffff0000" .. ERR_INV_FULL  .."|r")
				PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_error_sound), "Master")
			end
		end
		
		-- nebolo nic lootnute zobraz loot
		if slot.lootIcon then
			local shortLootName
			
			aLootFrame:Show()
			aLootFrame.DragBar:EnableMouse(false)
										
			-- create name			
			if L.config.name_short and (string_len(slot.lootName) > L.config.name_short_lenght) then
				if L.config.name_format == 0 then
					shortLootName =  string_gsub(slot.lootName, "%s?(.[\128-\191]*)%S+%s", "%1. ")
				elseif L.config.name_format == 1 then
					shortLootName = string_gsub(slot.lootName, "%s?(.[\128-\191]*)%S+%s", "%1.")
				elseif L.config.name_format == 2 then
					shortLootName =  string_gsub(slot.lootName, "%s(%a)%a*", " %1.")
				elseif L.config.name_format == 3 then
					shortLootName =  string_gsub(slot.lootName, "%s(%a)%a*", "%1.")
				elseif L.config.name_format == 4 then
					for n in string_gmatch(slot.lootName, '%S+') do
						shortLootName = n
						break
					end
				elseif L.config.name_format == 5 then
					for n in string_gmatch(slot.lootName, '%S+') do
						shortLootName = n
					end
				end
			else
				shortLootName = slot.lootName
			end
			
			if L.config.name_show or L.config.name_show_onlyMO then
				if L.config.name_color then
					slot.name:SetText(shortLootName)
					slot.name:SetTextColor(unpack(L.config.name_font_color))
				else
					--slot.name:SetTextColor(slot.color.r, slot.color.g, slot.color.b)
					slot.name:SetText(slot.color.hex .. shortLootName .."|r")
				end
			
				slot.nameTexture:ClearAllPoints()
				slot.nameTexture:SetPoint(L.config.name_font_point, slot, L.config.name_font_relativePoint, L.config.name_font_posun_X, L.config.name_font_posun_Y)
				slot.nameTexture:SetSize(slot.name:GetStringWidth() + 10, L.config.name_fontsize + 4)
				slot.name:SetAllPoints(slot.nameTexture)
			
				if L.config.name_texture_color then
					slot.nameTexture:SetVertexColor(unpack(L.config.name_texture_color_value))		
				else
					slot.nameTexture:SetVertexColor(slot.color.r, slot.color.g, slot.color.b, .6)
				end
						
				if L.config.name_show and not L.config.name_show_onlyMO then
					slot.nameTexture:Show()
					slot.name:Show()
				else
					slot.nameTexture:Hide()
					slot.name:Hide()
				end	
			end
			
			if L.config.lootFrame_itemPrice and slot.itemSellPrice then
				slot.sellPrice:SetText(GetMoneyShortString(slot.itemSellPrice))
				slot.sellPrice:Show()
			else
				slot.sellPrice:Hide()
			end
			
			if slot.lootQuantity and (slot.lootQuantity > 1) then
				slot.count:SetText(slot.lootQuantity)
				slot.count:Show()
			elseif slot.slotType == 2 then
				local coins = tonumber(string_match(slot.lootName, "(%d+)"))
				if coins and (coins > 0) then
					slot.count:SetText(coins)
					slot.count:Show()
				end
			else
				slot.count:Hide()
			end
							
			slot.icon:SetTexture(slot.lootIcon)
			--slot:SetBackdropColor(1, 0, 0, 0)
			slot:SetBackdropBorderColor(slot.color.r, slot.color.g, slot.color.b)
			slot:Enable()
		else
			slot.icon:SetTexture("Interface\\Icons\\INV_Misc_Herb_AncientLichen")
			slot.count:Hide()
			slot.border.shadow:Hide()
			slot:Disable()	
		end
		
		slot:SetSize(L.config.lootFrame_iconSize, L.config.lootFrame_iconSize)
		slot:Show()
		
		if slot.FadeIn:IsPlaying() then
			slot.FadeIn:Stop()
		end
		slot.FadeIn:Play()
		
		if slot.glow.animIn:IsPlaying() then
			slot.glow.animIn:Stop()
		end
		if L.config.loot_frame_flash then
			slot.glow:SetVertexColor(slot.color.r, slot.color.g, slot.color.b, 1)
			slot.glow.animIn:Play()
		end
		
	elseif aLootFrame.slots[slotIndex] then
		aLootFrame.slots[slotIndex]:Hide()
	end
end

local function OnLootOpened(...)
	local autoLoot, isFromItem = ...
	
	aLootFrame.isFromItem = isFromItem
	aLootFrame.autoLoot = autoLoot
	aLootFrame.lootReadyIsOpened = true
	aLootFrame.numLootItems = GetNumLootItems()
	
	if ( isFromItem ) then
		PlaySound(SOUNDKIT.UI_CONTAINER_ITEM_OPEN)
	end
	
	-- ak je zapnuty loot na mieste kurzora
	if C_CVar.GetCVarBool("lootUnderMouse") then
		local x, y = GetCursorPosition()
		x = x / aLootFrame:GetEffectiveScale()
		y = y / aLootFrame:GetEffectiveScale()
		aLootFrame:ClearAllPoints()
		aLootFrame:SetPoint("TOPLEFT", nil, "BOTTOMLEFT", x - (L.config.lootFrame_iconSize / 2), y + (L.config.lootFrame_iconSize / 2))
		aLootFrame:Raise()	
	end
		
	if aLootFrame.numLootItems > 0 then
		for i = 1, aLootFrame.numLootItems do
			OpenSlot(i)
		end
	end
	
	anchorSlots()
end

local function OnLootSlotCleared(...)
	local slotIndex = ...
	local slot = aLootFrame.slots[slotIndex]
	if slot and slot.opened then
		if aLootFrame:IsShown() and slot:IsShown() then
			slot:Hide()
			anchorSlots()
		end
		slot.opened = false
	end
end

function OnShowLootFrameBase(self)
	local numLootItems = GetNumLootItems()
	if( numLootItems == 0 ) then
		PlaySound(SOUNDKIT.LOOT_WINDOW_OPEN_EMPTY)
	elseif( IsFishingLoot() ) then
		PlaySound(SOUNDKIT.FISHING_REEL_IN)
	else
		PlaySound(SOUNDKIT.SOULBINDS_OPEN_UI)
	end
end

function OnHideLootFrameBase(self)
	CloseLoot()
	--StaticPopup_Hide('CONFIRM_LOOT_DISTRIBUTION')
	--MasterLooterFrame:Hide()	
end

local function OnAddonLoaded(self, ...)
	local name = ...
	if name == _ then
		if not _G["aLootCFG"] then _G["aLootCFG"] = L.defaults end
		L.config = _G["aLootCFG"]

		for key, value in pairs(L.defaults) do
			if L.config[key] == nil then
				L.config[key] = value
			end
		end
		
		L.ItemsTable = ItemsTable or {}
		ItemsTable = L.ItemsTable
		
		LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(_, L.options)
		L.basic_page = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(_, "aLoot", nil, "basic_page")
		
		-- lootFrame				
		L.config.unlock_lootFrame = false
		aLootFrame.DragBar:EnableMouse(false)
		aLootFrame.Text:SetText(nil)
		aLootFrame:SetBackdropColor(0, 0, 0, 0)
		aLootFrame:SetBackdropBorderColor(0, 0, 0, 0)
		
		aLootFrame.slots = {}
		aLootFrame:SetScript("OnShow", OnShowLootFrameBase)
		aLootFrame:SetScript("OnHide", OnHideLootFrameBase)
		
		_G.LootFrame:UnregisterAllEvents()
		_G.AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST")
		
		table_insert(_G.UISpecialFrames, "aLootFrame")
		hooksecurefunc(_G.MasterLooterFrame, 'Hide', _G.MasterLooterFrame.ClearAllPoints)
		
		-- infoFrame
		L.config.unlock_infoFrame = false
		aLootInfoFrame.Text:SetText(nil)
		aLootInfoFrame:SetBackdropColor(0, 0, 0, 0)
		aLootInfoFrame:SetBackdropBorderColor(0, 0, 0, 0)
				
		aLootInfoFrame.infoFrameActiveFrames = CreateFramePool("Button", infoFrameBase, "aLootInfoFramePool", ResetInfoFrameFunction)
		aLootInfoFrame.infoFrameActiveFrames.last = nil
		aLootInfoFrame.inFront = 0
		
		-- flashFrame
		L.config.unlock_flashFrame = false
		aLootFlashFrame.DragBar:EnableMouse(false)		
		
		-- minimap icon
		L.miniMapIconSave = miniMapIconSave or {}
		miniMapIconSave = L.miniMapIconSave
		
		local LDB = LibStub("LibDataBroker-1.1"):NewDataObject( _ , {
			type = "data source",
			text = "aLoot Addon",
			icon = 4359252,
			OnClick = function(self, button)
				if _G.InterfaceOptionsFrame:IsShown() then
					_G.InterfaceOptionsFrame:Hide()
				else
					InterfaceOptionsFrame_OpenToCategory("aLoot")
					InterfaceOptionsFrame_OpenToCategory(L.basic_page)
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("|cFF00FF00aLoot|r")
			end,
		})
		
		LibStub("LibDBIcon-1.0"):Register(_, LDB, miniMapIconSave)
		if L.config.aLoot_minimap then
			LibStub("LibDBIcon-1.0"):Show("aLoot")
		end
			
		-- Slash handler
		SLASH_ALOOT1 = "/aloot"
		SlashCmdList["ALOOT"] = function(msg)
			local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
			if ( cmd == "test" ) then
				for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
					local numSlots = GetContainerNumSlots(bagID)
					for slot = 1, numSlots do
						local itemLoc = ItemLocation:CreateFromBagAndSlot(bagID, slot)
						if itemLoc:IsValid() then
							local id = C_Item.GetItemID(itemLoc)
							if id == 190738 then
								local itemFrame
								if Nivaya then
									itemFrame = Nivaya:GetButton(bagID, slot)
								elseif ElvUI then
									itemFrame =  _G["ElvUI_ContainerFrameBag" .. bagID .. "Slot" .. slot];
								else
									itemFrame =  _G["ContainerFrame" .. (bagID + 1) .. "Item" .. (GetContainerNumSlots(bagID) + 1) - slot];
								end
									
								if (itemFrame) then
									--print(bagID,slot)
									itemFrame.NewItemTexture:SetAtlas("bags-glow-white");
									itemFrame.NewItemTexture:Show();
									if (not itemFrame.flashAnim:IsPlaying() and not itemFrame.newitemglowAnim:IsPlaying()) then
										itemFrame.flashAnim:Play();
										itemFrame.newitemglowAnim:Play();
									end
								end
							end
						end
					end
				end
				
			else
				--LibStub('AceConfigDialog-3.0'):Open("aLoot")
				InterfaceOptionsFrame_OpenToCategory("aLoot")
				InterfaceOptionsFrame_OpenToCategory(L.basic_page)
			end
		end
		
		if IsAddOnLoaded(_) then
			self:UnregisterEvent('ADDON_LOADED')
			self.ADDON_LOADED = nil
		end
	end
end

function aLootFrameSlotMixin:OnEnter()
	if L.config.toolTip then
		if ( self.slotType == LOOT_SLOT_CURRENCY ) then
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 10, -10)
			GameTooltip:SetLootCurrency(self:GetID())
			CursorUpdate(self)
		else
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 10, -10)
			GameTooltip:SetLootItem(self:GetID())
			CursorUpdate(self)
		end
	end
	
	if L.config.name_show and L.config.name_show_onlyMO then
		self.nameTexture:Show()
		self.name:Show()
	end
	
	if L.config.mouseover_scale > 1 then
		SetFrameScale(self, L.config.lootFrame_iconSize, L.config.lootFrame_iconSize, 1, L.config.mouseover_scale)
	end
	
	if L.config.mouseover_color then
		self:SetBackdropBorderColor(unpack(L.config.mouseover_color_value))
	end
end

function aLootFrameSlotMixin:OnLeave()
	if L.config.toolTip then
		GameTooltip:Hide()
	end
	
	if L.config.name_show_onlyMO then
		self.nameTexture:Hide()
		self.name:Hide()
	end
		
	if L.config.mouseover_scale > 1 then
		SetFrameScale(self, L.config.lootFrame_iconSize, L.config.lootFrame_iconSize, L.config.mouseover_scale, 1)
	end
	
	if self.lootQuality > 1 then
		self:SetBackdropBorderColor(self.color.r, self.color.g, self.color.b)
	else
		self:SetBackdropBorderColor(0, 0, 0)
	end
		
	ResetCursor()
end

function aLootFrameSlotMixin:OnClick(button)
	if self.lootName == "Test" then
		alootFrameMixin:UpdateNotifyChange("openLoot", true)
		
	elseif button == "LeftButton" and not IsModifiedClick() then
		StaticPopup_Hide('CONFIRM_LOOT_DISTRIBUTION')
		LootSlot(self:GetID())
		
	elseif button == "LeftButton" and IsModifiedClick() then
		HandleModifiedItemClick(GetLootSlotLink(self:GetID()))
	
	elseif button == "RightButton" and not IsModifiedClick() and self.slotType == 1 then
		ToggleAllBags()
		
		local itemFrame
		for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
			local numSlots = GetContainerNumSlots(bagID)
			for slotID = 1, numSlots do
				if Nivaya then
					itemFrame = Nivaya:GetButton(bagID, slotID)
				elseif Bagnon then
					itemFrame =  _G["ContainerFrame" .. (bagID + 1) .. "Item" .. slotID + 1]
				elseif ElvUI then
					itemFrame =  _G["ElvUI_ContainerFrameBag" .. bagID .. "Slot" .. slotID]
				else
					itemFrame =  _G["ContainerFrame" .. (bagID + 1) .. "Item" .. (GetContainerNumSlots(bagID) + 1) - slotID]
				end
						
				if (itemFrame) then
					local slotLink = GetContainerItemLink(bagID, slotID)
					if slotLink and  slotLink == self.itemLink then
						--print(bagID, slotID)
						itemFrame.NewItemTexture:SetAtlas("bags-glow-white")
						itemFrame.NewItemTexture:Show()
						if (not itemFrame.flashAnim:IsPlaying() and not itemFrame.newitemglowAnim:IsPlaying()) then
							itemFrame.flashAnim:Play()
							itemFrame.newitemglowAnim:Play()
						end
					else
						if (itemFrame.flashAnim:IsPlaying()) then
							itemFrame.flashAnim:Stop()
						end
						if (itemFrame.newitemglowAnim:IsPlaying()) then
							itemFrame.newitemglowAnim:Stop()
						end
					end
				end
			end
		end

		--[[if NivayacBniv_Bag then
			NivayacBniv_Bag.Search:DoSearch(self.lootName)
		elseif ElvUI then
			ElvUI[1]:GetModule('Bags'):SetSearch(self.lootName)
		else
			SetItemSearch(self.lootName)
		end
		]]--
		
		
	elseif button == "RightButton" and IsModifiedClick() and L.config.lootFrame_delete then
		local tmpTime = GetTime()
		if self.lastClick == nil then
			self.lastClick = tmpTime
		elseif (tmpTime - .3  > self.lastClick) then
			self.lastClick = tmpTime
		else
			self.lastClick = tmpTime
			
			LootSlot(self:GetID())
			
			for bagID = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
				local numSlots = GetContainerNumSlots(bagID)
				for slotID = 1, numSlots do
					local slotLink = GetContainerItemLink(bagID, slotID)
					if slotLink and slotLink == self.itemLink then
						PickupContainerItem(bagID, slotID)
						DeleteCursorItem()
					end
				end		
			end
			alootFrameMixin:ShowFlashFrame(nil, "|cffff0000All "..self.lootName.." has been deleted in your bags.|r")
			--alootFrameMixin:AddFrameToPool(133670, "|cffff0000All "..self.lootName.." has been deleted in your bags.|r")
		end
	end
end


function aLootFrameSlotMixin:OnShow()
	if GameTooltip:IsOwned(self) then
		GameTooltip:SetOwner(self,"ANCHOR_BOTTOMRIGHT", 10, -10)
		GameTooltip:SetLootItem(self:GetID())
		CursorOnUpdate(self)
	end
end

function alootFrameMixin:ConfigUpdate()
	if aLootFrame.numLootItems and aLootFrame.numLootItems > 0 then
		for i = 1, aLootFrame.numLootItems do
			aLootFrame.slots[i]:SetSize(L.config.lootFrame_iconSize, L.config.lootFrame_iconSize)
			aLootFrame.slots[i].nameTexture:SetTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, L.config.name_texture))
			aLootFrame.slots[i].name:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.name_font), L.config.name_fontsize,L.config.name_font_flag)
			if aLootFrame.slots[i].lootName == "Test" then
				OpenSlot(i, true)
				anchorSlots()
			else
				OpenSlot(i)
			end
		end
		anchorSlots()
	end
end

-- iba takto sa da prepisat hodnota config z XML suboru
function alootFrameMixin:UpdateNotifyChange(key, value)
	if key == "aLootFrame" then
		L.config.unlock_lootFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "aLootFlashFrame" then
		L.config.unlock_flashFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "aLootInfoFrame" then
		L.config.unlock_infoFrame = value
		print("|cffFF0000LOCK|r and save frame position\n")
		LibStub("AceConfigRegistry-3.0"):NotifyChange("aLoot")
	elseif key == "openLoot" then
		if aLootFrame:IsShown() and aLootFrame.slots[1] and aLootFrame.slots[1]:IsShown() then
			aLootFrame.slots[1]:Hide()
			aLootFrame.numLootItems = 0
			aLootFrame:Hide()
		else
			OpenSlot(1, true)
			anchorSlots()
		end
	end	
end

function alootFrameMixin:OnEvent(event, ...)
	--print(event, ...)
	if ( event == "ADDON_LOADED" ) then
		OnAddonLoaded(self, ...)
			
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		local errorType, message = ...
		if L.config.flashFrame and aLootFrame.lootReadyIsOpened then
			if MultiCheck(message, ERR_INV_FULL, ERR_ITEM_MAX_COUNT, ERR_LOOT_SLOT, ERR_LOOT_STUNNED, ERR_LOOT_NO_UI, ERR_LOOT_WHILE_INVULNERABLE, ERR_NO_LOOT) then
				if not aLootFlashFrame.Anime:IsPlaying() then
					local icon
					for slotIndex = aLootFrame.numLootItems, 1, -1 do
						if aLootFrame.slots[slotIndex].opened and not aLootFrame.slots[slotIndex]:IsShown() then
							OpenSlot(slotIndex, nil, true)
							anchorSlots()
							icon = aLootFrame.slots[slotIndex].lootIcon
						end
					end
					alootFrameMixin:ShowFlashFrame(icon or nil, "|cffff0000" .. message  .."|r")
					PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.flash_error_sound), "Master")
				end
			end
		end

	elseif ( event == "LOOT_READY" and L.config.speedLoot and not IsModifiedClick("AUTOLOOTTOGGLE") ) then
		OnLootReady(...)
		
	elseif ( event == "LOOT_OPENED" and (not L.config.speedLoot or (L.config.speedLoot and IsModifiedClick("AUTOLOOTTOGGLE")))) then
		OnLootOpened(...)
				
	elseif ( event == "LOOT_SLOT_CLEARED" ) then
		OnLootSlotCleared(...)
				
	elseif ( event == "LOOT_CLOSED" ) then
		OnLootClosed()
						
	elseif ( event == "OPEN_MASTER_LOOT_LIST" ) then
		MasterLooterFrame_Show(_G.LootFrame.selectedLootButton)
		--ToggleDropDownMenu(1, nil, GroupLootDropDown, LootFrame.selectedLootButton, 0, 0)		--LootFrame.selectedSlot
			
	elseif ( event == "UPDATE_MASTER_LOOT_LIST" ) then
		if _G.LootFrame.selectedLootButton then
			MasterLooterFrame_UpdatePlayers()
		end
		--MasterLooterFrame_UpdatePlayers()
	end
end

function alootFrameMixin:OnLoad()
	for _, event in pairs({
		"ADDON_LOADED",
		"UI_ERROR_MESSAGE",
		"LOOT_READY",
		"LOOT_OPENED",
		"LOOT_SLOT_CLEARED",
		"LOOT_CLOSED",
		"OPEN_MASTER_LOOT_LIST",
		"UPDATE_MASTER_LOOT_LIST",
	}) do self:RegisterEvent(event) end
--	alootFrameMixin = nil
end
